'' =================================================================================================
''
''   File....... jm_qmc5883l__wip.spin
''   Purpose....
''   Author..... Jon McPhalen
''               Copyright (c) 2025 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 05 JAN 2025
''
'' =================================================================================================


con { timing }

  _xinfreq = 5_000_000                                          ' use 5MHz crystal
  _clkmode = xtal1 + pll16x                                     ' x16 = 80MHz

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate


con { fixed io pins }

  PGM_RX   = 31  { I }                                          ' serial / programming
  PGM_TX   = 30  { O }

  EE_SDA   = 29  { I/O }                                        ' i2c / eeprom
  EE_SCL   = 28  { I/O }


con { app io pins }

  SDA2     = 17  { I/O }                                        ' secondary I2C bus
  SCL2     = 16  { I/O }


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  prng : "jm_prng"                                              '   pseudo-random number generation
  io   : "jm_io"                                                '   simple io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal

' * uses cog when loaded


dat { pre-initialized }


var { globals }


pub main

  setup
  wait_for_terminal(true, 250)

  term.fstr0(string("QMC5883 Developemnt\r"))

  if (present)
    term.fstr0(string("-- device found\r"))
    term.fstr1(string("-- ID: 0x%2x\r"), device_id)
  else
    term.fstr0(string("-- device not found\r"))
    pgm_stop

  repeat
    term.fstr0(string("\002\000\004"))
    read_xyz
    term.fstr1(string("X Axis: %6d\r"), get_x)
    term.fstr1(string("Y Axis: %6d\r"), get_y)
    term.fstr1(string("Z Axis: %6d\r"), get_z)
    time.pause(100)

' whoops

  pgm_stop


pri pgm_stop

  repeat
    waitcnt(0)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer

  io.start(0, 0)                                                ' clear all pins (master cog)

  term.tstart(BR_TERM)                                          ' start serial for terminal *

  start(SCL2, SDA2)                                             ' start QMC5883


pub wait_for_terminal(clear, delay)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)
    time.pause(delay #> 0)


con { object }

  QMC5883_WR = %0001_101_0
  QMC5883_RD = %0001_101_1


con { QMC5883L regs }

  R_X_LSB  = $00
  R_X_MSB  = $01
  R_Y_LSB  = $02
  R_Y_MSB  = $03
  R_Z_LSB  = $04
  R_Z_MSB  = $05

  R_CTRL1  = $09
  R_CTRL2  = $0A
  R_SETRST = $0B

  R_DEVID  = $0D


obj

  i2c : "jm_i2c"


var

  long  xaxis
  long  yaxis
  long  zaxis


pub null

'' This is not an application


pub start(sclpin, sdapin) : result

  i2c.setupx(sclpin, sdapin)
  waitcnt(cnt + (clkfreq / 20))

  if (present)
    write8(R_SETRST, $01)
    write8(R_CTRL1,  $11)                                       ' continuous, 10Hz, 8g, os512
    return true

  else
    return false


pub present : result

  i2c.start
  result := i2c.write(QMC5883_WR)
  i2c.stop

  return (result == i2c#ACK)


pub soft_reset

  write8(R_CTRL2, $80)


pub device_id : id

  return read8(R_DEVID)


pub ready : result

  return ((read8($06) & 1) == 1)


pub read_xyz | tx, ty, tz

  longfill(@tx, 0, 3)                                           ' clear temp registers

  i2c.start                                                     ' read signed 16-bit values
  i2c.write(QMC5883_WR)
  i2c.write(R_X_LSB)
  i2c.start
  i2c.write(QMC5883_RD)
  tx.byte[0] := i2c.read(i2c#ACK)
  tx.byte[1] := i2c.read(i2c#ACK)
  ty.byte[0] := i2c.read(i2c#ACK)
  ty.byte[1] := i2c.read(i2c#ACK)
  tz.byte[0] := i2c.read(i2c#ACK)
  tz.byte[1] := i2c.read(i2c#NAK)
  i2c.stop

  xaxis := ~~tx                                                 ' convert to longs
  yaxis := ~~ty
  zaxis := ~~tz


pub get_x : result

  return xaxis


pub get_y : result

  return yaxis


pub get_z : result

  return zaxis


pri write8(reg, value)

  i2c.start
  i2c.write(QMC5883_WR)
  i2c.write(reg)
  i2c.write(value)
  i2c.stop


pri read8(reg) : value

  i2c.start
  i2c.write(QMC5883_WR)
  i2c.write(reg)
  i2c.start
  i2c.write(QMC5883_RD)
  result := i2c.read(i2c#NAK)
  i2c.stop


pri read16(reg) : result

'' Read signed 16-bit value starting at reg

  i2c.start
  i2c.write(QMC5883_WR)
  i2c.write(reg)
  i2c.start
  i2c.write(QMC5883_RD)
  result.byte[0] := i2c.read(i2c#ACK)
  result.byte[1] := i2c.read(i2c#NAK)
  i2c.stop

  ~~result                                                      ' extend sign from bit 15


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}